/*
	ili9341 SPI Hardware Treiber
	Markus Horbach
	23.08.2016
	fr STM32CubeMX  / Keil 5.x
*/
#include "ili9341.h"

uint8_t *zeiger; 				// Puffer fr externe Zeiger auf Bilddaten
uint8_t scanline[480];	// Puffer zum schnellen Lschen vom Bildschirm

// DMA Verwaltung Start
enum {
	SPI2_DMA_WAIT,			// 0
	SPI2_DMA_COMPLETE,		// 1
	SPI2_DMA_ERROR			// 2
};

volatile uint16_t SPI2_DMA_flag = SPI2_DMA_COMPLETE;

void HAL_SPI_TxCpltCallback(SPI_HandleTypeDef *hspi)
{
	SPI2_DMA_flag = SPI2_DMA_COMPLETE;
	//printf("SPI2_DMA_CPLT\n");
}

void HAL_SPI_ErrorCallback(SPI_HandleTypeDef *hspi)
{
	SPI2_DMA_flag = SPI2_DMA_ERROR;
	printf("SPI2_DMA_ERR\n");
}
// DMA Verwaltung Ende

void ili9341_HardReset(void){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
	HAL_GPIO_WritePin(ili9341_Reset_Port, ili9341_Reset_Pin, GPIO_PIN_SET);
	HAL_Delay(1);
	HAL_GPIO_WritePin(ili9341_Reset_Port, ili9341_Reset_Pin, GPIO_PIN_RESET);
	HAL_Delay(10);
	HAL_GPIO_WritePin(ili9341_Reset_Port, ili9341_Reset_Pin, GPIO_PIN_SET);
	HAL_Delay(120);
}
void ili9341_SoftReset(void){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
	ili9341_WriteCommand(ili9341_SOFTWARE_RESET);
}
uint16_t ili9341_getXres(void){
	return ili9341_Xres;
}
uint16_t ili9341_getYres(void){
	return ili9341_Yres;
}
void ili9341_SetCMDFlag(void){
	HAL_GPIO_WritePin(ili9341_DC_Port, ili9341_DC_Pin, GPIO_PIN_RESET);
}
void ili9341_SetDataFlag(void){
	HAL_GPIO_WritePin(ili9341_DC_Port, ili9341_DC_Pin, GPIO_PIN_SET);
}
void ili9341_WriteCommand(uint8_t ili_cmd){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
	ili9341_SetCMDFlag();
	HAL_SPI_Transmit(ili9341_4Pin_SPI, &ili_cmd, 1, 1);
}
void ili9341_WriteData(uint8_t ili_data){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
	ili9341_SetDataFlag();
	HAL_SPI_Transmit(ili9341_4Pin_SPI, &ili_data, 1, 1);
}
void ili9341_ReadData(void){
}
void ili9341_SleepOn(void){
	ili9341_WriteCommand(ili9341_SLEEP_ON);
}
void ili9341_SleepOff(void){
	ili9341_WriteCommand(ili9341_SLEEP_OFF);
}
void ili9341_DisplayOn(void){
	ili9341_WriteCommand(ili9341_DISPLAY_ON);
}
void ili9341_DisplayOff(void){
	ili9341_WriteCommand(ili9341_DISPLAY_OFF);
}
void ili9341_DisplayInversionOn(void){
	ili9341_WriteCommand(ili9341_DISPLAY_INV_ON);
}
void ili9341_DisplayInversionOff(void){
	ili9341_WriteCommand(ili9341_DISPLAY_INV_OFF);
}
void ili9341_PixelFormatSet_565(void){
	ili9341_WriteCommand(ili9341_PIXEL_FORMAT);
	ili9341_WriteData(0x55); // 16 bit Format
}
void ili9341_PixelFormatSet_666(void){
	ili9341_WriteCommand(ili9341_PIXEL_FORMAT);
	ili9341_WriteData(0x66); // 18 bit Format
}
void ili9341_Init(void){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
	ili9341_HardReset();
	HAL_Delay(10);
	
	ili9341_WriteCommand(ili9341_POWERA);	// Power Control A
	ili9341_WriteData(0x39);
	ili9341_WriteData(0x2C);
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x34);
	ili9341_WriteData(0x02);
	
	ili9341_WriteCommand(ili9341_POWERB); // Power Control B
	ili9341_WriteData(0x00);
	ili9341_WriteData(0xC1);
	ili9341_WriteData(0x30);
	
	ili9341_WriteCommand(ili9341_DTCA);	// Driver Timing Control A
	ili9341_WriteData(0x85);
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x78);

	ili9341_WriteCommand(ili9341_DTCB);	// Driver Timing ControlB
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x00);
	
	ili9341_WriteCommand(ili9341_POWER_SEQ); // Power On Sequence Control
	ili9341_WriteData(0x64);
	ili9341_WriteData(0x03);
	ili9341_WriteData(0x12);
	ili9341_WriteData(0x81);

	ili9341_WriteCommand(ili9341_PRC);	// Pump Ratio Control
	ili9341_WriteData(0x20);
	
	ili9341_WriteCommand(ili9341_POWER1);	// Power Control1
	ili9341_WriteData(0x23);
	
	ili9341_WriteCommand(ili9341_POWER2);	// Power Control2
	ili9341_WriteData(0x10);
	
	ili9341_WriteCommand(ili9341_VCOM1);	// VMC Control1
	ili9341_WriteData(0x3E);
	ili9341_WriteData(0x28);
	
	ili9341_WriteCommand(ili9341_VCOM2);	// VMC Control2
	ili9341_WriteData(0x86);
	
	ili9341_WriteCommand(ili9341_MEM_ACCESS_CTRL);	// Memory Access Control
	ili9341_WriteData(0x48);
	
	ili9341_WriteCommand(ili9341_PIXEL_FORMAT); // Pixel Format
	ili9341_WriteData(0x55); // 16 bit Format
	//ili9341_WriteData(0x66); // 18 bit Format
	
	ili9341_WriteCommand(ili9341_FRC_NORMAL);	// Frame Rate Control
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x1B);
	
	ili9341_WriteCommand(ili9341_DFC);	// Display FunctionControl
	ili9341_WriteData(0x08);
	ili9341_WriteData(0x82);
	ili9341_WriteData(0x27);
		
	ili9341_WriteCommand(ili9341_3GAMMA_EN);	// 3Gamma Function Disable
	ili9341_WriteData(0x00);
	
	ili9341_WriteCommand(ili9341_COLUMN_ADDR); // Column Adress
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x00);
	ili9341_WriteData(0xEF);
	
	ili9341_WriteCommand(ili9341_PAGE_ADDR); // Page Address
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x01);
	ili9341_WriteData(0x3F);
	
	ili9341_WriteCommand(ili9341_NORMAL_MODE_ON); // Normal Mode On
	
	ili9341_WriteCommand(ili9341_GAMMA);	// Gamma Curve selected
	ili9341_WriteData(0x01);
	
	ili9341_WriteCommand(ili9341_PGAMMA);	//Positive Gamma Correction
	ili9341_WriteData(0x0F);
	ili9341_WriteData(0x31);
	ili9341_WriteData(0x2B);
	ili9341_WriteData(0x0C);
	ili9341_WriteData(0x0E);
	ili9341_WriteData(0x08);
	ili9341_WriteData(0x4E);
	ili9341_WriteData(0xF1);
	ili9341_WriteData(0x37);
	ili9341_WriteData(0x07);
	ili9341_WriteData(0x10);
	ili9341_WriteData(0x03);
	ili9341_WriteData(0x0E);
	ili9341_WriteData(0x09);
	ili9341_WriteData(0x00);
	
	ili9341_WriteCommand(ili9341_NGAMMA);  // Negative Gamma Correction
	ili9341_WriteData(0x00);
	ili9341_WriteData(0x0E);
	ili9341_WriteData(0x14);
	ili9341_WriteData(0x03);
	ili9341_WriteData(0x11);
	ili9341_WriteData(0x07);
	ili9341_WriteData(0x31);
	ili9341_WriteData(0xC1);
	ili9341_WriteData(0x48);
	ili9341_WriteData(0x08);
	ili9341_WriteData(0x0F);
	ili9341_WriteData(0x0C);
	ili9341_WriteData(0x31);
	ili9341_WriteData(0x36);
	ili9341_WriteData(0x0F);
	
	ili9341_WriteCommand(ili9341_SLEEP_OFF); // Exit sleep
	HAL_Delay(120);
	ili9341_WriteCommand(ili9341_DISPLAY_ON); // Display On
}

void ili9341_ClearScreen(uint16_t colour){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		
	ili9341_SetNormalMode();
	for(uint16_t x=0;x<(ili9341_Xres*2);x++){
		scanline[x] = colour;
	}
	ili9341_WriteCommand(ili9341_GRAM);
	ili9341_SetDataFlag();	
	
	zeiger= &scanline[0];
	for(uint16_t y=0;y<ili9341_Yres;y++){
		while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		SPI2_DMA_flag=SPI2_DMA_WAIT;
		HAL_SPI_Transmit_DMA(ili9341_4Pin_SPI, zeiger, 480);
	}	
}
void ili9341_SendImage(uint16_t *image, uint16_t res_x, uint16_t res_y){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		
	ili9341_WriteCommand(ili9341_GRAM);
	ili9341_SetDataFlag();
	zeiger = (uint8_t *)image;

	for(uint16_t y=0;y<res_y;y++){
		for(uint16_t x=0;x<res_x;x++){
			zeiger++;
			HAL_SPI_Transmit(ili9341_4Pin_SPI, zeiger, 1, 1);
			zeiger--;
			HAL_SPI_Transmit(ili9341_4Pin_SPI, zeiger, 1, 1);
			zeiger= zeiger+2;
		}
	}
}

void ili9341_SendImageDMA(uint16_t *image, uint16_t res_x, uint16_t res_y){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}

	ili9341_WriteCommand(ili9341_GRAM);
	ili9341_SetDataFlag();
	zeiger = (uint8_t *)image;
	//zeiger--;											// soll DMA Fehler beheben
	for(uint16_t y=0;y<res_y;y++){
		while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		SPI2_DMA_flag = SPI2_DMA_WAIT;
		HAL_SPI_Transmit_DMA(ili9341_4Pin_SPI, zeiger, res_x*2);
		zeiger = zeiger + res_x*2;
	}
}

void ili9341_SetFrameMode(uint16_t x1, uint16_t x2,uint16_t y1, uint16_t y2){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		
	ili9341_WriteCommand(ili9341_COLUMN_ADDR);
	ili9341_WriteData((uint8_t)(x1/0x100));
	ili9341_WriteData((uint8_t)x1);
	ili9341_WriteData((uint8_t)(x2/0x100));
	ili9341_WriteData((uint8_t)x2);
	ili9341_WriteCommand(ili9341_PAGE_ADDR);
	ili9341_WriteData((uint8_t)(y1/0x100));
	ili9341_WriteData((uint8_t)y1);
	ili9341_WriteData((uint8_t)(y2/0x100));
	ili9341_WriteData((uint8_t)y2);
	ili9341_WriteCommand(ili9341_PARTIAL_MODE_ON);
}

void ili9341_ClearFrame(uint8_t color[], uint16_t res_x, uint16_t res_y){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		
	ili9341_WriteCommand(ili9341_GRAM);
	ili9341_SetDataFlag();
	for(uint16_t y=0;y<res_y+1;y++){
		for(uint16_t x=0;x<res_x+1;x++){
			HAL_SPI_Transmit(ili9341_4Pin_SPI, &color[0], 2, 1);
		}
	}
}

void ili9341_SetNormalMode(){
	while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		
	ili9341_WriteCommand(ili9341_COLUMN_ADDR);
	ili9341_WriteData((uint8_t)0);
	ili9341_WriteData((uint8_t)0);
	ili9341_WriteData((uint8_t)0);
	ili9341_WriteData((uint8_t)0xEF);
	ili9341_WriteCommand(ili9341_PAGE_ADDR);
	ili9341_WriteData((uint8_t)0);
	ili9341_WriteData((uint8_t)0);
	ili9341_WriteData((uint8_t)0x1);
	ili9341_WriteData((uint8_t)0x3F);
	ili9341_WriteCommand(ili9341_NORMAL_MODE_ON);
}

void ili9341_printf_12x16(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor){
	for(uint16_t cnt=0;cnt<anzahl;cnt++){
		//while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		ili9341_SetFrameMode(ili9341_Xfont_12x16*pos_x+cnt*ili9341_Xfont_12x16,ili9341_Xfont_12x16*pos_x+11+cnt*ili9341_Xfont_12x16,ili9341_Yfont_12x16*pos_y,ili9341_Yfont_12x16*pos_y+15);
		ili9341_SendImageDMA(font_ascii_12x16_fillCharBuffer(*characters, bgcolor, fcolor), ili9341_Xfont_12x16, ili9341_Yfont_12x16);
		characters++;
	}
}

void ili9341_printf_24x32(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor){
	for(uint16_t cnt=0;cnt<anzahl;cnt++){
		while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		ili9341_SetFrameMode(ili9341_Xfont_24x32*pos_x+cnt*ili9341_Xfont_24x32,ili9341_Xfont_24x32*pos_x+23+cnt*ili9341_Xfont_24x32,ili9341_Yfont_24x32*pos_y,ili9341_Yfont_24x32*pos_y+31);
		ili9341_SendImageDMA(font_ascii_24x32_fillCharBuffer(*characters, bgcolor, fcolor), ili9341_Xfont_24x32, ili9341_Yfont_24x32);
		characters++;
	}
}

void ili9341_printf_20x32(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor){
	for(uint16_t cnt=0;cnt<anzahl;cnt++){
		while(SPI2_DMA_flag != SPI2_DMA_COMPLETE){}
		ili9341_SetFrameMode(ili9341_Xfont_20x32*pos_x+cnt*ili9341_Xfont_20x32,ili9341_Xfont_20x32*pos_x+19+cnt*ili9341_Xfont_20x32,ili9341_Yfont_20x32*pos_y,ili9341_Yfont_20x32*pos_y+31);
		ili9341_SendImageDMA(font_ascii_20x32_fillCharBuffer(*characters, bgcolor, fcolor), ili9341_Xfont_20x32, ili9341_Yfont_20x32);
		characters++;
	}
}
