/*
	ili9341 SPI Hardware Treiber
	
	Markus Horbach
	23.08.2016
	
	fr STM32CubeMX  / Keil 5.x
*/
#include <stdlib.h>

#include "stm32f4xx_hal.h"
#include "dma.h"
#include "spi.h"
#include "gpio.h"
#include "colours.h"
#include "font_ascii_12x16.h"
#include "font_ascii_24x32.h"

// Hardware Schnittstelle, anzupassen an eigene Hardwarekonfiguration
#define ili9341_4Pin_SPI 		&hspi5
#define ili9341_Reset_Pin		GPIO_PIN_7
#define ili9341_Reset_Port		GPIOA
#define ili9341_DC_Pin 			GPIO_PIN_13
#define ili9341_DC_Port 		GPIOD
#define ili9341_DMA_TX 			&hdma_spi5_tx
//#define ili9341_DMA_RX 			&hdma_spi5_rx

// TFT Eigenschaften Grafik/Text
#define ili9341_Xres 		 240 // X resolution TFT display
#define ili9341_Yres 		 320 // Y resolution TFT display
#define ili9341_colorbits 16 // default color depth TFT display

#define ili9341_Xfont_12x16 		12 // char X font size
#define ili9341_Yfont_12x16			16 // char Y font sizen
#define ili9341_Xchar_12x16 		20 // number of chars in one row ( Xres/Xfont )
#define ili9341_Ychar_12x16 		20 // number of chars in one column ( Yres/Yfont)

#define ili9341_Xfont_24x32 		24 // char X font size
#define ili9341_Yfont_24x32			32 // char Y font sizen
#define ili9341_Xchar_24x32 		10 // number of chars in one row ( Xres/Xfont )
#define ili9341_Ychar_24x32 		10 // number of chars in one column ( Yres/Yfont)

#define ili9341_Xfont_20x32 		20 // char X font size
#define ili9341_Yfont_20x32			32 // char Y font sizen
#define ili9341_Xchar_20x32 		12 // number of chars in one row ( Xres/Xfont )
#define ili9341_Ychar_20x32 		10 // number of chars in one column ( Yres/Yfont)

// Command defines
#define ili9341_NOP             		 0x00   // NOP
#define ili9341_SOFTWARE_RESET       0x01   // Software Reset
#define ili9341_READ_DISP_INFO       0x04   // Read Display Information
#define ili9341_READ_DISP_STATUS     0x09   // Read Display Status
#define ili9341_READ_DISP_POWER_MODE 0x0A   // Read Display Power Mode
#define ili9341_READ_DISP_MADCTL     0x0B   // Read Display MADCTR
#define ili9341_READ_DISP_PIX_FORMAT 0x0C   // Read Display Pixel Format
#define ili9341_READ_DISP_IMG_Format 0x0D   // Read Display Image Format
#define ili9341_READ_DISP_SIG_MODE   0x0E   // Read Display Signal Mode
#define ili9341_READ_DISP_SELF_DIAG  0x0F   // Read Display Self Diagnostic Result
#define ili9341_SLEEP_ON             0x10   // Sleep on
#define ili9341_SLEEP_OFF            0x11   // Sleep off
#define ili9341_PARTIAL_MODE_ON      0x12   // Partial Mode
#define ili9341_NORMAL_MODE_ON       0x13   // Normal Mode
#define ili9341_DISPLAY_INV_OFF      0x20   // Display Inversion Off
#define ili9341_DISPLAY_INV_ON       0x21   // Display Inversion On
#define ili9341_GAMMA                0x26   // Gamma
#define ili9341_DISPLAY_OFF          0x28   // Display off
#define ili9341_DISPLAY_ON           0x29   // Display on
#define ili9341_COLUMN_ADDR          0x2A   // Column address
#define ili9341_PAGE_ADDR            0x2B   // Page address
#define ili9341_GRAM                 0x2C   // GRAM
#define ili9341_COLOR_SET            0x2D   // Color Set
#define ili9341_MEMORY_READ          0x2E   // Memory Read
#define ili9341_PARTIAL_AREA         0x30   // Partial Area
#define ili9341_VERT_SCROLL_DEF      0x33   // Vertical Scrolling Definition
#define ili9341_TEAR_EFFECT_OFF      0x34   // Tearing Effect Off
#define ili9341_TEAR_EFFECT_ON       0x35   // Tearing Effect On
#define ili9341_MEM_ACCESS_CTRL      0x36   // Memory Access Control
#define ili9341_VERT_SCROLL_ADDR     0x37   // Vertical Scroll Start Address
#define ili9341_IDLE_MODE_OFF        0x38   // Idle Mode Off
#define ili9341_IDLE_MODE_ON         0x39   // Idle Mode On
#define ili9341_PIXEL_FORMAT         0x3A   // Pixel Format
#define ili9341_WRITE_MEM_CONT       0x3C   // Write Memory Continue
#define ili9341_READ_MEM_CONT        0x3E   // Read Memory Continue
#define ili9341_SET_TEAR_SCANLINE    0x44   // Set Tear Scanline
#define ili9341_GET_SCANLINE         0x45   // Get Scanline
#define ili9341_WRITE_DISP_BRIGHT    0x51   // Write Display Brightness
#define ili9341_READ_DISP_BRIGHT     0x52   // Read Display Brightness
#define ili9341_WCD                  0x53   // Write Control Display
#define ili9341_RCD                  0x54   // Read Control Display
#define ili9341_WCABC                0x55   // Write Content Adaptive Brightness Control
#define ili9341_RCABC                0x56   // Read Content Adaptive Brightness Control
#define ili9341_WCABC_MIN_BRIGHT     0x5E   // Write_CABC Minimum Brightness
#define ili9341_RCABC_MIN_BRIGHT     0x5F   // Read_CABC Minimum Brightness

// Optionale Commandos Start - default=NOP
#define ili9341_RGB_INTERFACE        0xB0   // RGB Interface Signal Control
#define ili9341_FRC_NORMAL           0xB1   // Frame Rate Control Normal Mode
#define ili9341_FRC_IDLE             0xB2   // Frame Rate Control Idle Mode
#define ili9341_FRC_Partial          0xB3   // Frame Rate Control Partial Mode
#define ili9341_DISP_INV_CTRL        0xB4   // Display Inversion Mode Control 
#define ili9341_BPC                  0xB5   // Blanking Porch Control
#define ili9341_DFC                  0xB6   // Display Function Control
#define ili9341_ENTRY_MODE_SET       0xB7   // Entry Mode Set
#define ili9341_BACKLIGHT_CTRL_1     0xB8   // Backlight Control 1
#define ili9341_BACKLIGHT_CTRL_2     0xB9   // Backlight Control 2
#define ili9341_BACKLIGHT_CTRL_3     0xBA   // Backlight Control 3
#define ili9341_BACKLIGHT_CTRL_4     0xBB   // Backlight Control 4
#define ili9341_BACKLIGHT_CTRL_5     0xBC   // Backlight Control 5
#define ili9341_BACKLIGHT_CTRL_6     0xBD   // Backlight Control 6
#define ili9341_BACKLIGHT_CTRL_7     0xBE   // Backlight Control 7
#define ili9341_BACKLIGHT_CTRL_8     0xBF   // Backlight Control 8
#define ili9341_POWER1               0xC0   // Power Control 1
#define ili9341_POWER2               0xC1   // Power Control 2
#define ili9341_VCOM1                0xC5   // VCOM Control 1
#define ili9341_VCOM2                0xC7   // VCOM Control 2
#define ili9341_POWERA               0xCB   // Power control A
#define ili9341_POWERB               0xCF   // Power control B
#define ili9341_NV_MEM_WRITE         0xD0   // NV Memory Write
#define ili9341_NV_MEM_PROT_KEY      0xD1   // NV Memory Protection Key
#define ili9341_NV_MEM_STATUS_READ   0xD2   // NV Memory Status Read
#define ili9341_READ_ID4             0xD3   // Read ID 4
// Optionale Commandos Ende

#define ili9341_READ_ID1             0xDA   // Modules Manufacturer
#define ili9341_READ_ID2             0xDB   // LCD Module / Driver Version
#define ili9341_READ_ID3             0xDC   // LCD Module / Driver ID

// Optionale Commandos Start - default=NOP
#define ili9341_PGAMMA               0xE0   // Positive Gamma Correction
#define ili9341_NGAMMA               0xE1   // Negative Gamma Correction
#define ili9341_DIGITAL_GAMMA_CTRL1  0xE2   // Digital Gamma Control 1
#define ili9341_DIGITAL_GAMMA_CTRL2  0xE3   // Digital Gamma Control 2
#define ili9341_DTCA                 0xE8   // Driver timing control A
#define ili9341_DTCB                 0xEA   // Driver timing control B
#define ili9341_POWER_SEQ            0xED   // Power on sequence register
#define ili9341_3GAMMA_EN            0xF2   // 3 Gamma enable register
#define ili9341_INTERFACE            0xF6   // Interface control register
#define ili9341_PRC                  0xF7   // Pump ratio control register 
// Optionale Commandos Ende

// Deklaration der Funktionen
void ili9341_HardReset(void); 							// GPIO Pin HardReset auslsen
void ili9341_SoftReset(void);
uint16_t ili9341_getXres(void); 						// maximale Bildschirmauflsung X
uint16_t ili9341_getYres(void); 						// maximale Bildschirmauflsung Y
void ili9341_SetCMDFlag(void); 							// GPIO Pin DC auf CMD setzen
void ili9341_SetDataFlag(void);							// GPIO Pin DC auf DATA setzen
void ili9341_WriteCommand(uint8_t ili_cmd);	// ein CMD Byte senden
void ili9341_WriteData(uint8_t ili_data); 	// ein DATA Byte senden
void ili9341_ReadData(void); 								// ein DATA Byte empfangen -> dummy function
void ili9341_SleepOn(void);
void ili9341_SleepOff(void);
void ili9341_DisplayOn(void);
void ili9341_DisplayOff(void);
void ili9341_DisplayInversionOn(void);
void ili9341_DisplayInversionOff(void);
void ili9341_PixelFormatSet_565(void);
void ili9341_PixelFormatSet_666(void);
void ili9341_Init(void); 										// ILI9341 initialisieren mit RGB565 240x320 Pixel
void ili9341_SendFont(void); 								// Bildschirm mit ASCII Font fllen
void ili9341_SendText(uint8_t *characters); // Bildschirm mit Text fllen
void ili9341_ClearScreen(uint16_t colour); 	// Bildschirm mit Farbe fllen
void ili9341_SendImage(uint16_t *image, uint16_t res_x, uint16_t res_y); 	  // ein Bitmap Bild RGB565 anzeigen
void ili9341_SendImageDMA(uint16_t *image, uint16_t res_x, uint16_t res_y);   // ein Bitmap Bild RGB565 anzeigen mit DMA
void ili9341_SetFrameMode(uint16_t x1, uint16_t x2,uint16_t y1, uint16_t y2); // Bildschirmausschnitt definieren
void ili9341_ClearFrame(uint8_t color[], uint16_t res_x, uint16_t res_y); 	// Bildschirmausschnitt mit Farbe fllen
void ili9341_SetNormalMode(void); 							// ganzen Bildschirm auswhlen
void ili9341_printf_12x16(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor); // einen Text in einer Zeile an gegebener Position schreiben
void ili9341_printf_24x32(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor); // einen Text in einer Zeile an gegebener Position schreiben
void ili9341_printf_20x32(uint8_t *characters, uint16_t anzahl, uint16_t pos_x, uint16_t pos_y, uint16_t bgcolor, uint16_t fcolor); // einen Text in einer Zeile an gegebener Position schreiben

