/*
	LTDC FX - ntzliche Routinen fr den Framebuffer
	12.02.2017 Markus Horbach
	fr STM32CubeMX
*/

#include "ltdc_fx.h"

uint32_t LTDC_FX_frontbuffer0;	// Adressen fr Layer0
uint32_t LTDC_FX_backbuffer0;
uint32_t LTDC_FX_drawbuffer0;
uint32_t LTDC_FX_buffer00;
uint32_t LTDC_FX_buffer01;

uint32_t LTDC_FX_frontbuffer1;	// Adressen fr Layer1
uint32_t LTDC_FX_backbuffer1;
uint32_t LTDC_FX_drawbuffer1;
uint32_t LTDC_FX_buffer10;
uint32_t LTDC_FX_buffer11;

volatile uint16_t backbuffer_status = status_empty;
//volatile uint16_t drawbuffer_finished=0;

uint16_t syncs = 0;			// Variablen fr den Framecounter
uint16_t swaps = 0;
uint16_t fps = 0;
uint8_t str_fps[] = "69";
uint8_t *ptr_fps = &str_fps[0];

void HAL_LTDC_LineEventCallback(LTDC_HandleTypeDef *hltdc)
{
	UNUSED(hltdc);
	if(backbuffer_status == status_full){
		swaps++;
		LTDC_FX_SwapFrontBuffer();

	}

	syncs++;			// Framecounter aktualisieren
	if(syncs>59){
		fps=swaps;
		syncs=0;
		swaps=0;
	}
	str_fps[0] = fps/10+48;
	str_fps[1] = fps%10+48;
	LTDC_FX_printf_12x16Alpha(ptr_fps, 2, 64, 0, RGB_COL_WHITE);

	__HAL_LTDC_ENABLE_IT(hltdc, LTDC_IT_LI);
}

void HAL_LTDC_ReloadEventCallback(LTDC_HandleTypeDef *hltdc)
{
	//irq_VSYNC = 1;
}

void LTDC_FX_Init(void){
	LTDC_FX_frontbuffer0 = LTDC_FX_framebuffer0;				// Speicherbereiche fr die Bildpuffer definieren
	LTDC_FX_ClearBuffer(LTDC_FX_frontbuffer0, RGB_COL_BLACK);
	LTDC_FX_backbuffer0  = LTDC_FX_framebuffer1;
	LTDC_FX_ClearBuffer(LTDC_FX_backbuffer0, RGB_COL_BLACK);
	LTDC_FX_drawbuffer0  = LTDC_FX_framebuffer2;
	LTDC_FX_ClearBuffer(LTDC_FX_drawbuffer0, RGB_COL_BLACK);

	LTDC_FX_frontbuffer1 = LTDC_FX_framebuffer3;				// Speicherbereiche fr die Bildpuffer definieren
	LTDC_FX_ClearBuffer(LTDC_FX_frontbuffer1, RGB_COL_BLACK);
	LTDC_FX_backbuffer1  = LTDC_FX_framebuffer4;
	LTDC_FX_ClearBuffer(LTDC_FX_backbuffer1, RGB_COL_BLACK);
	LTDC_FX_drawbuffer1  = LTDC_FX_framebuffer5;
	LTDC_FX_ClearBuffer(LTDC_FX_drawbuffer1, RGB_COL_BLACK);

	HAL_LTDC_SetAddress(&hltdc, LTDC_FX_frontbuffer0 , 0);	// Frontbuffer dem LTDC zuweisen
	HAL_LTDC_SetAddress(&hltdc, LTDC_FX_frontbuffer1 , 1);

	HAL_LTDC_ProgramLineEvent(&hltdc, 480);					// V-Sync aktivieren
}
void LTDC_FX_ClearBuffer(uint32_t imgbuffer, uint16_t colour){
	  for(uint32_t data=0;data<(480/48);data++){
		  HAL_DMA_Start(&hdma_memtomem_dma2_stream2, colour , imgbuffer+data*2*800*48, 800*48); // Buffer mit BGcolor lschen
		  HAL_DMA_PollForTransfer(&hdma_memtomem_dma2_stream2, HAL_DMA_FULL_TRANSFER,1);
	  }
}
void LTDC_FX_SwapFrontBuffer(void){
	LTDC_FX_buffer00 = LTDC_FX_frontbuffer0;
	LTDC_FX_frontbuffer0 = LTDC_FX_backbuffer0;
	LTDC_FX_backbuffer0 = LTDC_FX_buffer00;
	HAL_LTDC_SetAddress(&hltdc, LTDC_FX_frontbuffer0 , 0);
	LTDC_FX_ClearBuffer(LTDC_FX_backbuffer0, RGB_COL_BLACK);
	backbuffer_status = status_empty;
}
void LTDC_FX_SwapDrawBuffer(void){
	while(backbuffer_status == status_full){}
	LTDC_FX_buffer01 = LTDC_FX_backbuffer0;
	LTDC_FX_backbuffer0 = LTDC_FX_drawbuffer0;
	LTDC_FX_drawbuffer0 = 	LTDC_FX_buffer01;
	backbuffer_status = status_full;
}
uint16_t LTDC_FX_DrawBufferEmpty(void){
	return backbuffer_status;
}
void LTDC_FX_DrawImage(uint16_t *ptr_image, uint16_t pic_resx, uint16_t pic_resy, uint16_t pic_posx, uint16_t pic_posy){
	  for(uint32_t data=0;data<pic_resy;data++){
		  HAL_DMA_Start(&hdma_memtomem_dma2_stream5, (uint32_t)(ptr_image+pic_resx*data) , LTDC_FX_drawbuffer0+pic_posx*2+pic_posy*2*800+data*2*800 , pic_resx);
		  HAL_DMA_PollForTransfer(&hdma_memtomem_dma2_stream5, HAL_DMA_FULL_TRANSFER,1);
	  }
}
void LTDC_FX_DrawImageAlpha(uint16_t *ptr_image, uint16_t pic_resx, uint16_t pic_resy, uint16_t pic_posx, uint16_t pic_posy){
	  uint16_t* ptr_sdram;

	  ptr_sdram = ((uint16_t *)LTDC_FX_drawbuffer0+pic_posx+LTDC_FX_resx*pic_posy);

	  for(uint16_t y=pic_posy;y<(pic_posy+pic_resy);y++){
		  for(uint16_t x=0;x<pic_resx;x++){
			  if(*ptr_image!=0){
				*ptr_sdram = *ptr_image;
			  }
			  	ptr_sdram++;
			  	ptr_image++;
		  }
		  ptr_sdram+=(LTDC_FX_resx-pic_resx);
	  }
}
void LTDC_FX_printf_12x16(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy){// 66 x 30 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImage(font_ascii_12x16_fillCharBuffer(*chars, RGB_COL_BLACK , RGB_COL_WHITE), 12, 16, char_posx*12+x*12, char_posy*16);
		chars++;
	}
}
void LTDC_FX_printf_12x16Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol){// 66 x 30 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImageAlpha(font_ascii_12x16_fillCharBuffer(*chars, RGB_COL_BLACK , charcol), 12, 16, char_posx*12+x*12, char_posy*16);
		chars++;
	}
}
void LTDC_FX_printf_24x32(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy){// 33 x 15 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImage(font_ascii_24x32_fillCharBuffer(*chars, RGB_COL_BLACK , RGB_COL_WHITE), 24, 32, char_posx*24+x*24, char_posy*32);
		chars++;
	}
}
void LTDC_FX_printf_24x32Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol){// 33 x 15 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImageAlpha(font_ascii_24x32_fillCharBuffer(*chars, RGB_COL_BLACK , charcol), 24, 32, char_posx*24+x*24, char_posy*32);
		chars++;
	}
}
void LTDC_FX_printf_20x32(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy){// 40 x 15 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImage(font_ascii_20x32_fillCharBuffer(*chars, RGB_COL_BLACK , RGB_COL_WHITE), 20, 32, char_posx*20+x*20, char_posy*32);
		chars++;
	}
}
void LTDC_FX_printf_20x32Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol){// 40 x 15 Zeichen
	for(uint16_t x=0;x<count;x++){
		LTDC_FX_DrawImageAlpha(font_ascii_20x32_fillCharBuffer(*chars, RGB_COL_BLACK , charcol), 20, 32, char_posx*20+x*20, char_posy*32);
		chars++;
	}
}


