/*
	LTDC FX - ntzliche Routinen fr den Framebuffer
	12.02.2017 Markus Horbach
	fr STM32CubeMX
*/

#include <stdlib.h>
#include "stm32f4xx_hal.h"
#include "colours.h"
#include "font_ascii_12x16.h"
#include "font_ascii_24x32.h"
#include "ltdc.h"

#define LTDC_FX_resx			(uint16_t)800
#define LTDC_FX_resy			(uint16_t)480
#define LTDC_FX_pixel			(uint32_t)384000		// 800x480pixel
#define LTDC_FX_char_12x16_resx	(uint16_t)66			// @font 12x16
#define LTDC_FX_char_12x16_resy	(uint16_t)30			// @font 12x16
#define LTDC_FX_char_24x32_resx	(uint16_t)33			// @font 24x32
#define LTDC_FX_char_24x32_resy	(uint16_t)15			// @font 24x32
#define LTDC_FX_char_20x32_resx	(uint16_t)40			// @font 20x32
#define LTDC_FX_char_20x32_resy	(uint16_t)15			// @font 20x32

#define LTDC_FX_bufsize			(uint32_t)0xbb800		// 800x480x2 - 750kByte, 768000Byte
#define LTDC_FX_framebuffer0	(uint32_t)0xd0000000	// 800x480x2 - 750kB, end buffer 0xd00bb800-1
#define LTDC_FX_framebuffer1	(uint32_t)0xd00bb800	// 800x480x2 - 750kB, end buffer 0xd0177000-1
#define LTDC_FX_framebuffer2	(uint32_t)0xd0177000	// 800x480x2 - 750kB, end buffer 0xd0232800-1
#define LTDC_FX_framebuffer3	(uint32_t)0xd0232800	// 800x480x2 - 750kB, end buffer 0xd02EE000-1
#define LTDC_FX_framebuffer4	(uint32_t)0xd02EE000	// 800x480x2 - 750kB, end buffer 0xd03A9800-1
#define LTDC_FX_framebuffer5	(uint32_t)0xd03A9800	// 800x480x2 - 750kB, end buffer 0xd0465000-1
#define LTDC_FX_framebufferend	(uint32_t)0xd0800000	// 8x124x1024 - 8MB, end buffer 0xd0800000-1

#define status_full				(uint16_t)0
#define status_empty			(uint16_t)1

#define LTDC_FX_fontbuffer_12x16  (uint32_t)0xd0232800
#define LTDC_FX_fontsize_12x16    (uint32_t)0x18000		// 12x16x2 - 384Byte*256char - 96kB

#define LTDC_FX_fontbuffer_24x32  (uint32_t)0xd0177000
#define LTDC_FX_fontsize_24x32    (uint32_t)0x18000		// 24x32x2 - 1536Byte*256char - 384kB

#define LTDC_FX_fontbuffer_20x32  (uint32_t)0xd0177000
#define LTDC_FX_fontsize_20x32    (uint32_t)0x18000		// 20x32x2 - 1280Byte*256char - 320kB

void LTDC_FX_Init(void);
void LTDC_FX_ClearBuffer(uint32_t imgbuffer, uint16_t colour);
void LTDC_FX_SwapFrontBuffer(void);
void LTDC_FX_SwapDrawBuffer(void);
uint16_t LTDC_FX_DrawBufferEmpty(void);
void LTDC_FX_DrawImage(uint16_t *ptr_image, uint16_t pic_resx, uint16_t pic_resy, uint16_t pic_posx, uint16_t pic_posy);
void LTDC_FX_DrawImageAlpha(uint16_t *ptr_image, uint16_t pic_resx, uint16_t pic_resy, uint16_t pic_posx, uint16_t pic_posy);
void LTDC_FX_printf_12x16(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy);
void LTDC_FX_printf_12x16Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol);
void LTDC_FX_printf_24x32(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy);
void LTDC_FX_printf_24x32Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol);
void LTDC_FX_printf_20x32(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy);
void LTDC_FX_printf_20x32Alpha(uint8_t *chars, uint16_t count, uint16_t char_posx, uint16_t char_posy, uint16_t charcol);


