/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * Copyright (c) 2017 STMicroelectronics International N.V. 
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted, provided that the following conditions are met:
  *
  * 1. Redistribution of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  * 3. Neither the name of STMicroelectronics nor the names of other 
  *    contributors to this software may be used to endorse or promote products 
  *    derived from this software without specific written permission.
  * 4. This software, including modifications and/or derivative works of this 
  *    software, must execute solely and exclusively on microcontroller or
  *    microprocessor devices manufactured by or for STMicroelectronics.
  * 5. Redistribution and use of this software other than as permitted under 
  *    this license is void and will automatically terminate your rights under 
  *    this license. 
  *
  * THIS SOFTWARE IS PROVIDED BY STMICROELECTRONICS AND CONTRIBUTORS "AS IS" 
  * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
  * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
  * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
  * SHALL STMICROELECTRONICS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
  * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f4xx_hal.h"
#include "adc.h"
#include "dma.h"
#include "dma2d.h"
#include "fatfs.h"
#include "ltdc.h"
#include "sdio.h"
#include "spi.h"
#include "gpio.h"
#include "fmc.h"

/* USER CODE BEGIN Includes */
#include "colours.h"
#include "font_ascii_12x16.h"
#include "font_ascii_24x32.h"
#include "sdram_fx.h"
#include "ili9341.h"
#include "ltdc_fx.h"
#include "Katrin_Bauerfeind_240x320_BGR.h"
#include "nora_tschirner_240x320_BGR.h"
#include "zooey_deschanel_240x320_BGR.h"
#include "Lala_Beach_800x480_BGR.h"

/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
uint16_t L2_alpha=0;
uint8_t L2_dir = 1;

uint16_t joystick=0;
uint16_t iliready=0;
HAL_SD_CardInfoTypedef meineSDcard;
HAL_SD_CardInfoTypedef *ptr_meineSDcard = &meineSDcard;
HAL_SD_ErrorTypedef meineSDcardFault;

FATFS driveFATFS;
FATFS *ptr_driveFATFS = &driveFATFS;
FIL fileFATFS;
FIL *ptr_fileFATFS = &fileFATFS;

uint16_t ADC_Temp[4];
uint16_t *ptr_ADC_Temp = &ADC_Temp[0];

uint8_t meinIntWert[6]  ={"000000"};
uint8_t meinFloatWert[7]={"0000.00"};
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
void Error_Handler(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
void flash2mem(uint16_t* flash, uint16_t*sram, uint32_t size){
	for(uint32_t cnt=0;cnt<size;cnt++){
		*sram = *flash;
		sram++;
		flash++;
	}
}

uint8_t* float2string(float wert){
	uint32_t iWert = (uint32_t)(wert*100);

	meinIntWert[0] = (uint8_t)(( iWert / 100000)+48);
	meinIntWert[1] = (uint8_t)(((iWert % 100000)/10000)+48);
	meinIntWert[2] = (uint8_t)(((iWert % 10000)/1000)+48);
	meinIntWert[3] = (uint8_t)(((iWert % 1000)/100)+48);
	meinIntWert[4] = (uint8_t)(((iWert % 100)/10)+48);
	meinIntWert[5] = (uint8_t)(( iWert % 10)+48);
	if(meinIntWert[0] == 48){
		meinIntWert[0]=' ';
	}
	for(uint16_t x=1;x<5;x++){
		if(meinIntWert[x-1] == ' ' && meinIntWert[x]=='0'){
			meinIntWert[x]=' ';
		}
	}

	meinFloatWert[0] = meinIntWert[0];
	meinFloatWert[1] = meinIntWert[1];
	meinFloatWert[2] = meinIntWert[2];
	meinFloatWert[3] = meinIntWert[3];
	meinFloatWert[5] = meinIntWert[4];
	meinFloatWert[6] = meinIntWert[5];
	return (uint8_t *)&meinFloatWert;
}

uint8_t* int2string(uint32_t wert){
	meinIntWert[0] = (uint8_t)(( wert / 100000)+48);
	meinIntWert[1] = (uint8_t)(((wert % 100000)/10000)+48);
	meinIntWert[2] = (uint8_t)(((wert % 10000)/1000)+48);
	meinIntWert[3] = (uint8_t)(((wert % 1000)/100)+48);
	meinIntWert[4] = (uint8_t)(((wert % 100)/10)+48);
	meinIntWert[5] = (uint8_t)(( wert % 10)+48);
	if(meinIntWert[0] == 48){
		meinIntWert[0]=' ';
	}
	for(uint16_t x=1;x<5;x++){
		if(meinIntWert[x-1] == ' ' && meinIntWert[x]=='0'){
			meinIntWert[x]=' ';
		}
	}
	return (uint8_t *)&meinIntWert;
}

uint8_t* ADC2Temp(uint16_t adc){
	float cal_h = 0.0;
	float cal_l = 0.0;
	float cal_x = 0.0;

	cal_h = (float)*(uint16_t *)0x1FF0F44E; // Kalibrierungswert 110 Grad C
	cal_l = (float)*(uint16_t *)0x1FF0F44C; // Kalibrierungswert 30 Grad C
	cal_x = (110-30)/(cal_h - cal_l);

	return float2string(cal_x*ADC_Temp[3]-272.72);
}
/* USER CODE END 0 */

int main(void)
{

  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* Configure the system clock */
  SystemClock_Config();

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_DMA_Init();
  MX_SPI5_Init();
  MX_FMC_Init();
  MX_LTDC_Init();
  MX_DMA2D_Init();
  MX_SDIO_SD_Init();
  MX_FATFS_Init();
  MX_ADC1_Init();

  /* USER CODE BEGIN 2 */
  SDRAM_Initialization_sequence(REFRESH_COUNT);
  font_ascii_12x16_Init();
  font_ascii_24x32_Init();
  init_Katrin_Bauerfeind_240x320_BGR();
  init_nora_tschirner_240x320_BGR();
  init_zooey_deschanel_240x320_BGR();
  init_Lala_Beach_800x480_BGR();



  // set GPIO ili9341 start
  HAL_GPIO_WritePin(ILI9341_Reset_GPIO_Port,ILI9341_Reset_Pin,GPIO_PIN_SET);
  HAL_GPIO_WritePin(ILI9341_RDX_GPIO_Port,ILI9341_RDX_Pin,GPIO_PIN_SET);
  HAL_GPIO_WritePin(MEMS_SPI_SW_CS_GPIO_Port,MEMS_SPI_SW_CS_Pin,GPIO_PIN_SET);
  // set GPIO ili9341 end
  LTDC_FX_Init();
  ili9341_Init();
  ili9341_ClearScreen(RGB_COL_BLACK);
/*
  FRESULT fr  = 0;
    uint32_t btr= 0;
    uint8_t buff[20];
    void *ptr_buff = &buff[0];
    uint32_t br = 0;
    int32_t *ptr_br = &br;

    fr = f_mount(ptr_driveFATFS, "", 1);
    if(fr == 0){
  	  ili9341_printf_12x16((uint8_t *)"0OK", 3, 0, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }else{
  	  ili9341_printf_12x16(int2string(fr), 6, 0, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }

    //fr = f_setlabel("STM32sdCard");

    fr = f_open(ptr_fileFATFS, (char *)"README.TXT", FA_CREATE_ALWAYS | FA_WRITE);
    if(fr == 0){
  	  ili9341_printf_12x16((uint8_t *)"1OK", 3,6, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }else{
  	  ili9341_printf_12x16(int2string(fr), 6, 6, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }

   // fr = f_write(ptr_fileFATFS, get_nora_tschirner_240x320_BGR(),240*320*2,ptr_br);

        f_puts("____________________\n" ,ptr_fileFATFS);
    for(uint16_t x=0;x<40;x++){
    	  f_puts("|-> Hallo~Welt ! <-|\n" ,ptr_fileFATFS);
    }
    	  f_puts("--------------------\n" ,ptr_fileFATFS);

    fr = f_close(ptr_fileFATFS);
    if(fr == 0){
  	  ili9341_printf_12x16((uint8_t *)"2OK", 3,12, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }else{
  	  ili9341_printf_12x16(int2string(fr), 6, 12, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }

    fr = f_mount(0, "", 1);
    if(fr == 0){
  	  ili9341_printf_12x16((uint8_t *)"3OK", 3,18, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }else{
  	  ili9341_printf_12x16(int2string(fr), 6, 18, 16, RGB_COL_BLACK, RGB_COL_WHITE);
    }
  // Datei anlegen und beschreiben Ende
*/
  ili9341_printf_20x32((uint8_t *)"SystemStatus", 12, 0, 0, RGB_COL_BLACK, RGB_COL_WHITE);
  iliready=1;

  int16_t y0=0;
  int16_t x0=0;

  int16_t y1=80;
  int16_t x1=240;

  int16_t y2=160;
  int16_t x2=480;

  uint16_t dirx0=0;
  uint16_t diry0=0;

  uint16_t dirx1=0;
  uint16_t diry1=0;

  uint16_t dirx2=0;
  uint16_t diry2=0;

  HAL_LTDC_SetAddress(&hltdc, (uint32_t)get_Lala_Beach_800x480_BGR() , 1);

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
	  while(LTDC_FX_DrawBufferEmpty() == status_full){}

	  LTDC_FX_DrawImage(get_Katrin_Bauerfeind_240x320_BGR(), 240, 320, x0, y0);
//	  LTDC_FX_DrawImage(get_nora_tschirner_240x320_BGR(), 240, 320, x1, y1);
//	  LTDC_FX_DrawImage(get_zooey_deschanel_240x320_BGR(), 240, 320, x2, y2);

//	  LTDC_FX_DrawImage(get_Katrin_Bauerfeind_240x320_BGR(), 240, 320, (x0+270)%800, y0);
	  LTDC_FX_DrawImage(get_nora_tschirner_240x320_BGR(), 240, 320, (x1+270)%800, y1);
//	  LTDC_FX_DrawImage(get_zooey_deschanel_240x320_BGR(), 240, 320, (x2+270)%800, y2);

//	  LTDC_FX_DrawImage(get_Katrin_Bauerfeind_240x320_BGR(), 240, 320, (x0+540)%800, y0);
//	  LTDC_FX_DrawImage(get_nora_tschirner_240x320_BGR(), 240, 320, (x1+540)%800, y1);
	  LTDC_FX_DrawImage(get_zooey_deschanel_240x320_BGR(), 240, 320, (x2+540)%800, y2);

	  if(x0<(800-240-6)&&dirx0==0){x0+=6;}else{dirx0=1;}
	  if(x0>6 && dirx0==1)      {x0-=6;}else{dirx0=0;}
	  if(y0<(480-320-4)&&diry0==0){y0+=4;}else{diry0=1;}
	  if(y0>4 && diry0==1)      {y0-=4;}else{diry0=0;}

	  if(x1<(800-240-12)&&dirx1==0){x1+=12;}else{dirx1=1;}
	  if(x1>12 && dirx1==1)      {x1-=12;}else{dirx1=0;}
	  if(y1<(480-320-8)&&diry1==0){y1+=8;}else{diry1=1;}
	  if(y1>8 && diry1==1)      {y1-=8;}else{diry1=0;}

	  if(x2<(800-240-18)&&dirx2==0){x2+=18;}else{dirx2=1;}
	  if(x2>18 && dirx2==1)      {x2-=18;}else{dirx2=0;}
	  if(y2<(480-320-12)&&diry2==0){y2+=12;}else{diry2=1;}
	  if(y2>12 && diry2==1)      {y2-=12;}else{diry2=0;}

	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 1, 1, RGB_COL_RED);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 3, 2, RGB_COL_RED);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 5, 3, RGB_COL_YELLOW);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 7, 4, RGB_COL_YELLOW);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 9, 5, RGB_COL_GREEN);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 11, 6, RGB_COL_GREEN);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 13, 7, RGB_COL_ORANGE);

//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 15, 8, RGB_COL_ORANGE);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 17, 9, RGB_COL_BLUE);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 19, 10, RGB_COL_BLUE);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 21, 11, RGB_COL_NEARBLACK);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 23, 12, RGB_COL_NEARBLACK);
	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 25, 13, RGB_COL_PURPUR);
//	  LTDC_FX_printf_20x32Alpha((uint8_t *)"DASROTEMOPPED", 13, 27, 14, RGB_COL_PURPUR);

	  LTDC_FX_SwapDrawBuffer();

	  //joystick

	  if(HAL_GPIO_ReadPin(GPIOE, Joystick_Up_Pin)){ili9341_printf_20x32((uint8_t *)"X", 1, 5, 2, RGB_COL_BLACK, RGB_COL_GREEN);}else{ili9341_printf_20x32((uint8_t *)"O", 1, 5, 2, RGB_COL_BLACK, RGB_COL_GREY);} // up
	  if(HAL_GPIO_ReadPin(GPIOE, Joystick_Down_Pin)){ili9341_printf_20x32((uint8_t *)"X", 1, 5, 6, RGB_COL_BLACK, RGB_COL_GREEN);}else{ili9341_printf_20x32((uint8_t *)"O", 1, 5, 6, RGB_COL_BLACK, RGB_COL_GREY);} // down
	  if(HAL_GPIO_ReadPin(GPIOE, Joystick_Left_Pin)){ili9341_printf_20x32((uint8_t *)"X", 1, 3, 4, RGB_COL_BLACK, RGB_COL_GREEN);}else{ili9341_printf_20x32((uint8_t *)"O", 1, 3, 4, RGB_COL_BLACK, RGB_COL_GREY);} // left
	  if(HAL_GPIO_ReadPin(GPIOE, Joystick_Right_Pin)){ili9341_printf_20x32((uint8_t *)"X", 1, 7, 4, RGB_COL_BLACK, RGB_COL_GREEN);}else{ili9341_printf_20x32((uint8_t *)"O", 1, 7, 4, RGB_COL_BLACK, RGB_COL_GREY);} // right
	  if(HAL_GPIO_ReadPin(GPIOE, Joystick_Fire_Pin)){ili9341_printf_20x32((uint8_t *)"X", 1, 5, 4, RGB_COL_BLACK, RGB_COL_GREEN);}else{ili9341_printf_20x32((uint8_t *)"O", 1, 5, 4, RGB_COL_BLACK, RGB_COL_GREY);} // fire

	  HAL_LTDC_SetAlpha(&hltdc, L2_alpha, 1);

	  if(L2_dir == 1 && L2_alpha < 252 ){L2_alpha = L2_alpha+4;}
	  if(L2_alpha == 252){ L2_dir = 0;}

	  if(L2_dir == 0 && L2_alpha > 0){L2_alpha = L2_alpha-4;}
	  if(L2_alpha == 0){ L2_dir = 1;}
  }
  /* USER CODE END 3 */

}

/** System Clock Configuration
*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_PeriphCLKInitTypeDef PeriphClkInitStruct;

    /**Configure the main internal regulator output voltage 
    */
  __HAL_RCC_PWR_CLK_ENABLE();

  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 8;
  RCC_OscInitStruct.PLL.PLLN = 360;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 8;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Activate the Over-Drive mode 
    */
  if (HAL_PWREx_EnableOverDrive() != HAL_OK)
  {
    Error_Handler();
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5) != HAL_OK)
  {
    Error_Handler();
  }

  PeriphClkInitStruct.PeriphClockSelection = RCC_PERIPHCLK_LTDC;
  PeriphClkInitStruct.PLLSAI.PLLSAIN = 118;
  PeriphClkInitStruct.PLLSAI.PLLSAIR = 2;
  PeriphClkInitStruct.PLLSAIDivR = RCC_PLLSAIDIVR_2;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */

}

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
